@@ -2,6 +2,7 @@ from django.contrib import admin
 from django.utils.safestring import mark_safe
 from django.forms import Widget
 from django.contrib.admin import SimpleListFilter
+from django.db.models import Q, F, Count
 import mimetypes
 
 def set_title (title):
@@ -78,6 +79,11 @@ class NullFilter (SimpleListFilter):
         return queryset
 
 
+class StackedInline (admin.StackedInline):
+    can_delete = False
+    show_change_link = True
+
+
 # ModelAdmin -------------------------------------------
 class ModelAdmin (admin.ModelAdmin):
     image_width = 320
@@ -86,8 +92,8 @@ class ModelAdmin (admin.ModelAdmin):
     enable_delete = True
     enable_change = True
 
-    def __init__(self, *args, **kargs):
-        super().__init__(*args, **kargs)
+    list_per_page = 100
+    list_max_show_all = 200
 
     def has_add_permission(self, request, obj=None):
         return self.enable_add
@@ -98,9 +104,14 @@ class ModelAdmin (admin.ModelAdmin):
     def has_change_permission(self, request, obj=None):
         return self.enable_change
 
-    def get_filtered_queryset (self, request):
-        cl = self.get_changelist_instance (request)
-        return cl.get_queryset (request)
+    def before_changelist_view (self, queryset, context):
+        pass
+
+    def changelist_view (self, request, extra_context = None):
+        r = super ().changelist_view (request)
+        if hasattr (r, 'context_data') and 'cl' in r.context_data:
+            self.before_changelist_view (r.context_data ['cl'].queryset, r.context_data)
+        return r
 
     def formfield_for_dbfield (self, db_field, request, **kwargs):
         if 'widget' not in kwargs:
